class ParamDef(object):
	KEEPALIVE = 1
	AGENT_BOOT_ENV = 2
	CONFIGURE_UPDATE = 3
	CONFIGURE_GET = 4
	COMPO_VERSIONS = 5
	THREAD_LIST = 7
	THREAD_DETAIL = 8
	GET_ACTIVE_STACK = 9
	HEAP_HISTO = 10
	LOADED_CLASS_LIST = 11
	GET_ENV = 12
	SYSTEM_GC = 13
	SET_CONFIG = 14
	OPEN_SOCKET_LIST = 15
	LOADED_CLASS_DETAIL = 16
	LOADED_CLASS_REDEFINE = 17
	AGENT_JAR_LIST = 18
	AGENT_JAR_SAVE = 19
	AGENT_JAR_DELETE = 20
	RESET_STRING_SENT_MARK = 21
	AGENT_LOG_LIST = 22
	AGENT_LOG_READ = 23
	THREAD_CONTROL = 24
	GET_ACTIVE_TRANSACTION_LIST = 25
	GET_ACTIVE_TRANSACTION_DETAIL = 26
	GET_TOPOLOGY = 27
	INFRA_NET_STAT=28
	INFRA_PS_EF=29
	INFRA_DOCKER_LIST=30
	INFRA_REMOTE_CMD=33
	INFRA_PEER_LIST = 34
	INFRA_AGENT_UPDATE = 36
	AGENT_DUMP_LIST = 31
	AGENT_DUMP_READ = 32
	AGENT_STAT = 35
	JVM_THREAD_DUMP = 37
	JVM_HEAP_DUMP = 38
	KUBENETES = 39
	GET_ACTIVE_STATS = 40
	MODULE_DEPENDENCY = 101
	DBX_INFO = 201
	DBX_SETTINGS = 202
	DBX_USER_SCRIPT = 207
	DBX_SCRIPT_LIST = 208
	DBX_SCRIPT_SQL = 209
	DBX_KILL_SESSION = 210
	DBX_GET_ALERT = 211
	DBX_SET_ALERT = 212
	DBX_PLAN = 213
	DBX_SESSION_LIST = 214
	DBX_SET_CPUCORES = 215
	DBX_SESSION_STAT = 203
	DBX_SESSION_EVENT = 204
	DBX_TOPSTAT = 205
	DBX_TOPEVENT = 206
	CONFIG_DBC = 301
	METHOD_PERF_STAT = 302
	BLOCKING_DETECT = 303
	CW_INFO = 401
	CW_EVENTS = 402
	CW_CREDENTIAL = 403
	CW_REGIONS = 404
	CW_METRIC_NAMES = 405
	CW_AWS_RESOURCE_IDS = 406
	CW_STATISTICS = 407
	CW_UNITS = 408
	CW_METRICS = 409